package controller;

import java.io.IOException;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import model.Model;

import org.mybeans.dao.DAOException;

import databean.User;

import controller.Action;
import controller.AddEntryAction;
import controller.CreateUserAction;
import controller.DeleteEntryAction;
import controller.DeleteUserAction;
import controller.LoginAction;
import controller.LogoutAction;
import controller.LookupEntryAction;
import controller.ManageUsersAction;
import controller.ModifyEntryAction;
import controller.RemoteLoginAction;
import controller.SearchAction;
import controller.SetPasswordAction;
import controller.UpdateEntryAction;


public class Controller extends HttpServlet {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	Model model;
	
    public void init() throws ServletException {
        model = new Model(getServletConfig());

        Action.add(new AddEntryAction(model));
        Action.add(new CreateUserAction(model));
        Action.add(new DeleteEntryAction(model));
        Action.add(new DeleteUserAction(model));
        Action.add(new LoginAction(model));
        Action.add(new LogoutAction(model));
        Action.add(new LookupEntryAction(model));
        Action.add(new ManageUsersAction(model));
        Action.add(new ModifyEntryAction(model));
        Action.add(new RemoteLoginAction(model));
        Action.add(new SearchAction(model));
        Action.add(new SetPasswordAction(model));
        Action.add(new UpdateEntryAction(model));
    }

    public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        doGet(request,response);
    }

    public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        if (model.getRequireSSL() && !request.isSecure()) {
			// If we're required to use SSL and we're not on a secure connection, redirect
        	String host = request.getServerName();
        	String port = (request.getServerPort()==80) ? "" : ":8443";
            response.sendRedirect("https://"+host+port+request.getRequestURI());
            return;
        }

        try {
	        model.getLogDAO().write(request, "doGet: "+request.getRequestURI());
	        String nextPage = performTheAction(request);
	        sendToNextPage(nextPage,request,response);
        } catch (DAOException e) {
        	e.printStackTrace();
        	request.setAttribute("dbError",e.getMessage());
	        sendToNextPage("db-error.jsp",request,response);
        }
    }
    
    /*
     * Extracts the requested action and (depending on whether the user is logged in)
     * perform it (or make the user login).
     * @param request
     * @return the next page (the view)
     */
    private String performTheAction(HttpServletRequest request) {
        HttpSession session     = request.getSession(true);
        String      servletPath = request.getServletPath();
        User        user        = (User) session.getAttribute("user");
        String      actionName  = getActionName(servletPath);
        
        // Si el usuario no esta logueado y no es un usuario local, entonces lo enviamos a loguearse
        if (user == null && !isLocalIPAddress(request.getRemoteAddr())) {
			return Action.perform("remote-login",request);
        }
        
        // si es que no hay usuarios en el sistema, hacer este usuario (el cual es local, ya que sino ya estaria logueado)
        // el usuario inicial (presumiblemente el mismo)
        try {
        	int numUsers = model.getUserDAO().getCount();
        	if (numUsers == 0) return Action.perform("create-user",request);
        } catch (DAOException e) {
        	// Perhaps the DB is down
        	// Skip the initial user check and just proceed with the users requested
        	// (which will fail, also, if the database is down
        }
        
        if (actionName.equals("start")) {
        	// si el usuario es local o ya esta logueado, enviarlo a la pagina de search
			return Action.perform("search",request);
        }

      	// Dejar al usuario local o logueado hacer cualquier accion que el desee.
		return Action.perform(actionName,request);
    }
    
    /*
     * If nextPage is null, send back 404
     * If nextPage starts with a '/', redirect to this page.
     *    In this case, the page must be the whole servlet path including the webapp name
     * Otherwise dispatch to the page (the view)
     *    This is the common case
     * Note: If nextPage equals "image", we will dispatch to /image.  In the web.xml file, "/image"
     *    is mapped to the ImageServlet will which return the image bytes for display.
     */
    private void sendToNextPage(String nextPage, HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
    	// System.out.println("nextPage="+nextPage);
    	
    	if (nextPage == null) {
    		response.sendError(HttpServletResponse.SC_NOT_FOUND,request.getServletPath());
    		return;
    	}
    	
    	if (nextPage.charAt(0) == '/') {
    		String proto = request.isSecure() ? "https://" : "http://";
			String host  = request.getServerName();
			String port  = ":"+String.valueOf(request.getServerPort());
			if (port.equals(":80")) port = "";
			if (port.equals(":443")) port = "";
			String context = request.getContextPath();
			int lastSlash = context.lastIndexOf('/');
			String prefix = ( lastSlash==0 ? context : context.substring(0,lastSlash) );
			response.sendRedirect(proto+host+port+prefix+nextPage);
			return;
    	}
    	
   		RequestDispatcher d = request.getRequestDispatcher(nextPage);
   		d.forward(request,response);
    }

	/*
	 * Returns the path component after the last slash removing any "extension"
	 * if present.
	 */
    private String getActionName(String path) {
    	// 
        int slash = path.lastIndexOf('/');
        int dot = path.lastIndexOf('.');
        if (dot == -1) dot = path.length();
        return path.substring(slash+1,dot);
    }
    
    private boolean isLocalIPAddress(String addr) {
    	String localPrefix = model.getLocalNetPrefix();
    	if (localPrefix != null && addr.startsWith(localPrefix)) return true;

    	String localAddr = model.getLocalNetAddr();
    	if (localAddr != null && addr.equals(localAddr)) return true;
    	
    	return false;
    }
}
